@@ -301,7 +301,7 @@ async def replicate_manifest_lists(docker_registry_secure: DockerRegistrySecure)
                 )
 
 
-def test___init__(docker_registry_client_async: DockerRegistryClientAsync):
+async def test___init__(docker_registry_client_async: DockerRegistryClientAsync):
     """Test that the docker registry client can be instantiated."""
     assert docker_registry_client_async
 
@@ -389,6 +389,92 @@ async def test__get_credentials(
     assert result == auth
 
 
+async def test__get_proxy(docker_registry_client_async: DockerRegistryClientAsync):
+    """Test that a proxy can be retrieved."""
+    endpoints = ["endpoint0.com", "endpoint1.com", "127.0.0.1"]
+    proxies = {
+        "http": "http://http-proxy.com:1234",
+        "https": "https://https-proxy.com:2345",
+    }
+
+    for protocol, url in proxies.items():
+        for endpoint in endpoints:
+            # Empty
+            docker_registry_client_async.proxies = {}
+            docker_registry_client_async.proxy_no = []
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            assert result is None
+
+            # http, no https, not in no_proxy
+            docker_registry_client_async.proxies = {"http": proxies["http"]}
+            docker_registry_client_async.proxy_no = []
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            if protocol == "http":
+                assert result == proxies["http"]
+            else:
+                assert result is None
+
+            # no http, https, not in no_proxy
+            docker_registry_client_async.proxies = {"https": proxies["https"]}
+            docker_registry_client_async.proxy_no = []
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            if protocol == "https":
+                assert result == proxies["https"]
+            else:
+                assert result is None
+
+            # http, https, not in no_proxy
+            docker_registry_client_async.proxies = proxies
+            docker_registry_client_async.proxy_no = []
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            assert result == url
+
+            # http, no https, in no_proxy
+            docker_registry_client_async.proxies = {"http": proxies["http"]}
+            docker_registry_client_async.proxy_no = [endpoint]
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            assert result is None
+
+            # no http, https, in no_proxy
+            docker_registry_client_async.proxies = {"https": proxies["https"]}
+            docker_registry_client_async.proxy_no = [endpoint]
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            assert result is None
+
+            # http, https, in no_proxy
+            docker_registry_client_async.proxies = proxies
+            docker_registry_client_async.proxy_no = [endpoint]
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            assert result is None
+
+            # http, https, in no_proxy (full list)
+            docker_registry_client_async.proxies = proxies
+            docker_registry_client_async.proxy_no = endpoints
+            result = await docker_registry_client_async._get_proxy(
+                endpoint=endpoint, protocol=protocol
+            )
+            assert result is None
+            # ... with something different ...
+            result = await docker_registry_client_async._get_proxy(
+                endpoint="foobar", protocol=protocol
+            )
+            assert result == url
+
+
 @pytest.mark.parametrize(
     "endpoint,auth",
     [
@@ -626,7 +712,8 @@ async def test_delete_manifest(
 
 @pytest.mark.online
 async def test__get_blob_config(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image blobs (image configurations) can be retrieved."""
     manifest_data = await get_manifest_json(
@@ -659,7 +746,8 @@ async def test__get_blob_config(
 
 @pytest.mark.online
 async def test__get_blob_layer(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image blobs (layers) can be retrieved."""
     manifest_data = await get_manifest_json(
@@ -692,7 +780,8 @@ async def test__get_blob_layer(
 
 @pytest.mark.online
 async def test_get_blob(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image blobs (layers) can be retrieved."""
     manifest_data = await get_manifest_json(
@@ -716,7 +805,7 @@ async def test_get_blob(
 @pytest.mark.online
 async def test_get_blob_to_disk_async(
     docker_registry_client_async: DockerRegistryClientAsync,
-    known_good_image,
+    known_good_image: TypingKnownGoodImage,
     tmp_path: Path,
 ):
     """Test that the image blobs (image configurations) can be retrieved to disk asynchronously."""
@@ -753,7 +842,7 @@ async def test_get_blob_to_disk_async(
 @pytest.mark.online
 async def test_get_blob_to_disk_sync(
     docker_registry_client_async: DockerRegistryClientAsync,
-    known_good_image,
+    known_good_image: TypingKnownGoodImage,
     tmp_path: Path,
 ):
     """Test that the image blobs (image configurations) can be retrieved to disk synchronously."""
@@ -850,17 +939,16 @@ async def test__get_catalog(
     known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image catalog can be retrieved."""
-    image_name = ImageName.parse(
-        f"{known_good_image['image']}:{known_good_image['tag']}"
-    )
     LOGGER.debug("Retrieving catalog ...")
-    client_response = await docker_registry_client_async._get_catalog(image_name)
+    client_response = await docker_registry_client_async._get_catalog(
+        known_good_image["image_name"]
+    )
     assert client_response
     assert client_response.status == HTTPStatus.OK
 
     catalog = json.loads(await client_response.text())
     assert "repositories" in catalog
-    assert image_name.resolve_image() in catalog["repositories"]
+    assert known_good_image["image_name"].resolve_image() in catalog["repositories"]
 
 
 @pytest.mark.online
@@ -884,7 +972,8 @@ async def test_get_catalog(
 
 @pytest.mark.online
 async def test__get_manifest(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image manifests can be retrieved."""
     identifier_map = get_identifier_map(known_good_image)
@@ -902,11 +991,7 @@ async def test__get_manifest(
             assert digest == image_name.resolve_digest()
             assert digest == client_response.headers["Docker-Content-Digest"]
         else:
-            LOGGER.debug(
-                "Skipping digest check for: %s (%s)",
-                image_name,
-                identifier_map[image_name],
-            )
+            LOGGER.debug("Skipping digest check for: %s (%s)", image_name, media_type)
 
         assert json.loads(await client_response.text())
         assert client_response.headers["Content-Type"] == media_type
@@ -914,7 +999,8 @@ async def test__get_manifest(
 
 @pytest.mark.online
 async def test_get_manifest(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image manifests can be retrieved."""
     identifier_map = get_identifier_map(known_good_image)
@@ -973,7 +1059,7 @@ async def test_get_manifest_sanity_check():
 @pytest.mark.online
 async def test_get_manifest_to_disk_async(
     docker_registry_client_async: DockerRegistryClientAsync,
-    known_good_image,
+    known_good_image: TypingKnownGoodImage,
     tmp_path: Path,
 ):
     """Test that the image manifests can be retrieved to disk asynchronously."""
@@ -999,7 +1085,7 @@ async def test_get_manifest_to_disk_async(
 @pytest.mark.online
 async def test_get_manifest_to_disk_sync(
     docker_registry_client_async: DockerRegistryClientAsync,
-    known_good_image,
+    known_good_image: TypingKnownGoodImage,
     tmp_path: Path,
 ):
     """Test that the image manifests can be retrieved to disk synchronously."""
@@ -1024,7 +1110,8 @@ async def test_get_manifest_to_disk_sync(
 
 @pytest.mark.online
 async def test__get_tags(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image tags can be retrieved."""
     image_name = ImageName.parse(
@@ -1043,7 +1130,8 @@ async def test__get_tags(
 
 @pytest.mark.online
 async def test_get_tag_list(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image tags can be retrieved."""
     image_name = ImageName.parse(
@@ -1063,7 +1151,8 @@ async def test_get_tag_list(
 
 @pytest.mark.online
 async def test_get_tags(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image tags can be retrieved."""
     image_name = ImageName.parse(
@@ -1129,7 +1218,8 @@ async def test_get_version(
 
 @pytest.mark.online
 async def test__head_blob(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image blobs (image configurations) can be retrieved."""
     manifest_data = await get_manifest_json(
@@ -1156,7 +1246,8 @@ async def test__head_blob(
 
 @pytest.mark.online
 async def test_head_blob(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image blobs (image configurations) can be retrieved."""
     manifest_data = await get_manifest_json(
@@ -1188,7 +1279,8 @@ async def test_head_blob(
 
 @pytest.mark.online
 async def test__head_manifest(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image manifests existence can be checked."""
     image_names = get_identifier_map(known_good_image).keys()
@@ -1209,7 +1301,8 @@ async def test__head_manifest(
 
 @pytest.mark.online
 async def test_head_manifest(
-    docker_registry_client_async: DockerRegistryClientAsync, known_good_image
+    docker_registry_client_async: DockerRegistryClientAsync,
+    known_good_image: TypingKnownGoodImage,
 ):
     """Test that the image manifests existence can be checked."""
     image_names = get_identifier_map(known_good_image).keys()
